// student CRUD

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "get_int.h"

#define DEBUG 1

struct student {
  int id;
  char name[25];
  int exam01;
  int exam02;
  int exam03;
  struct student * next;
};

struct student * load_data( );

void add_new_student( struct student * );

void display_all_students( struct student * );
void display_one_student( struct student * );

int save_all_students( struct student * );
void save_next_student( struct student *, FILE * );

struct student * create_student( int, char [], int, int, int );
void append_student(struct student ** current, struct student *);
void free_all(struct student *);

int find_next_id(struct student * );

void delete_student(struct student **, int);

int main( void ) {
  struct student * roster = NULL;
  int choice = 0, student_choice = 0;

  //load the student data
  roster = load_data( );
  if( roster == NULL ) {
    printf("The students.txt data file could not be found\n");
    return 1;
  }

  //display all students (menu)
  printf("-----------------------------\n");
  display_all_students( roster );
  int last_id = find_next_id(roster);
  int add_new_student_option = last_id+1;
  int exit_option = last_id+2;
  printf("%d. add new student\n", add_new_student_option);
  printf("%d. exit\n", exit_option);
  printf("-----------------------------\n");

  //choose option
  printf("choose an option: ");
  choice = get_int(stdin);

  while( choice != exit_option ) {
    //handle menu choice
    if( choice >= 1 && choice <= last_id ) {
      //do something with students
      printf("1. update that student\n");
      printf("2. delete that student\n");

      printf("make a choice: ");
      student_choice = get_int(stdin);

      if(student_choice == 1) {
        //update
      } else if(student_choice == 2){
        //delete
        delete_student(&roster, choice);
      } else {
        printf("That wasn't a valid option\n");
      }

    } else if( choice == add_new_student_option ) {
      //add new student
      add_new_student( roster );
    }

    //display all students (menu)
    printf("-----------------------------\n");
    display_all_students( roster );
    last_id = find_next_id(roster);
    add_new_student_option = last_id+1;
    exit_option = last_id+2;
    printf("%d. add new student\n", add_new_student_option);
    printf("%d. exit\n", exit_option);
    printf("-----------------------------\n");

    //choose option
    printf("choose an option: ");
    choice = get_int(stdin);
  }

  if( save_all_students(roster) != 0 ) {
    printf("data could not be written to the file\n");
  }

  free_all(roster);
  return 0;
} //end main


void delete_student(struct student ** current, int id) {
  if(DEBUG) printf("deleting student %d\n", id);
  if( (*current)->id == id ) {
    if(DEBUG) printf("\tcurrent pointer -> %p\n", *current);
    struct student * old_head = *current;
    if(DEBUG) printf("\told_head pointer -> %p\n", *current);
    *current = old_head->next;
    if(DEBUG) printf("\tcurrent reset to pointer -> %p\n", *current);
    free(old_head);
  } else if( (*current)->next->id == id ) {
    struct student * next_student_after_delete = (*current)->next->next;
    free((*current)->next);
    (*current)->next = next_student_after_delete;
  } else {
    delete_student(&(*current)->next, id);
  }
}


void add_new_student(struct student * roster) {
    char name[25];

    printf("Enter the new students name: ");
    get_string( name, 25, stdin );
    printf("Enter the first exam score: ");
    int g1 = get_int(stdin);
    if( DEBUG ) printf("you entered: %d\n", g1);
    printf("Enter the second exam score: ");
    int g2 = get_int(stdin);
    if( DEBUG ) printf("you entered: %d\n", g2);
    printf("Enter the third exam score: ");
    int g3 = get_int(stdin);
    if( DEBUG ) printf("you entered: %d\n", g3);

    int current_max_id = find_next_id(roster);
    struct student * new_student = create_student(++current_max_id, name, g1, g2, g3);
    append_student(&roster, new_student);
}

//load the student data from file
struct student * load_data( ) {
  struct student * list = NULL;
  FILE * data_file = fopen("students.txt", "r");
  if(! data_file) {
    return NULL;
  }

  char name[25];
  int id, g1, g2, g3;
  while( fscanf(data_file, "%d,%25[^,],%d,%d,%d\n", &id, name, &g1, &g2, &g3) == 5 ) {
    struct student * new_student = create_student(id, name, g1, g2, g3);
    append_student(&list, new_student);
  }

  return list;
}

struct student * create_student( int id, char name[], int g1, int g2, int g3 ) {
  struct student * current = malloc(sizeof(struct student));
  current->id = id;
  strcpy(current->name, name);
  current->exam01 = g1;
  current->exam02 = g2;
  current->exam03 = g3;
  current->next = NULL;
  return current;
}

void append_student(struct student ** current, struct student * new_student) {
  if( DEBUG ) printf("adding: %s\n", new_student->name);
  if( *current == NULL ) {
    if( DEBUG ) printf("head is null, creating root node\n");
    *current = new_student;
  } else {
    if( (*current)->next != NULL ) {
      if( DEBUG ) printf("recurse\n");
      append_student(&(*current)->next, new_student);
    } else {
      if( DEBUG ) printf("appending new node\n");
      (*current)->next = new_student;
    }
  }
}

void free_all(struct student * current) {
  if( current->next != NULL ) {
    free_all(current->next);
  }

  if( DEBUG ) printf("freeing node with number: %s\n", current->name);
  free(current);
}


int find_next_id(struct student * current) {
  int current_max_id = 0;

  if( current->next != NULL ) {
    current_max_id = find_next_id(current->next);
  }

  if( current->id > current_max_id ) {
    return current->id;
  } else {
    return current_max_id;
  }
}


//display the student data from arrays
void display_all_students( struct student * current ) {
  //display column headers
  printf("%-5s%-25s%10s%10s%10s\n\n", "ID", "Name", "Exam1", "Exam2", "Exam3");
  display_one_student( current );
}

void display_one_student( struct student * current ) {
  if( current != NULL ) {
    printf("%-5d", current->id);
    printf("%-25s", current->name); //left justified student name
    printf("%10d", current->exam01);
    printf("%10d", current->exam02);
    printf("%10d\n", current->exam03);

    display_one_student(current->next);
  }
}

// save all students in the linked list to file
int save_all_students( struct student * current ) {
  FILE * data_file = fopen("students.txt", "w");

  if( ! data_file ) {
    return 1;
  }

  save_next_student( current, data_file );
  fclose(data_file);
  return 0;
}

void save_next_student( struct student * current, FILE * file ) {
  if( current != NULL ) {
    fprintf(file, "%d,%s,%d,%d,%d\n",current->id,current->name,current->exam01,current->exam02,current->exam03);
    save_next_student(current->next,file);
  }
}
